pub struct RedNode { /* private fields */ }Expand description
Red node - represents an AST node with position information This is the “Red” part of Red-Green Syntax Tree
Implementations
Sourceimpl RedNode
impl RedNode
Sourcepub fn new(green_id: GreenNodeId, offset: usize) -> Arc<Self>
pub fn new(green_id: GreenNodeId, offset: usize) -> Arc<Self>
Create a new red node from a green node ID
Sourcepub fn new_with_parent(
green_id: GreenNodeId,
offset: usize,
parent: Weak<RedNode>,
) -> Arc<Self>
pub fn new_with_parent( green_id: GreenNodeId, offset: usize, parent: Weak<RedNode>, ) -> Arc<Self>
Create a new red node with a parent reference
Sourcepub fn width(&self, arena: &GreenNodeArena) -> usize
pub fn width(&self, arena: &GreenNodeArena) -> usize
Get the width (length) of this node
Sourcepub fn end(&self, arena: &GreenNodeArena) -> usize
pub fn end(&self, arena: &GreenNodeArena) -> usize
Get the end position of this node
Sourcepub fn kind(&self, arena: &GreenNodeArena) -> Option<SyntaxKind>
pub fn kind(&self, arena: &GreenNodeArena) -> Option<SyntaxKind>
Get the syntax kind of this node
Sourcepub fn green_id(&self) -> GreenNodeId
pub fn green_id(&self) -> GreenNodeId
Get the underlying green node ID
Sourcepub fn children(self: &Arc<Self>, arena: &GreenNodeArena) -> Vec<Arc<RedNode>>
pub fn children(self: &Arc<Self>, arena: &GreenNodeArena) -> Vec<Arc<RedNode>>
Get children as red nodes with parent references
Sourcepub fn is_descendant_of(&self, ancestor: &RedNode) -> bool
pub fn is_descendant_of(&self, ancestor: &RedNode) -> bool
Check if this node is a descendant of another node
Sourcepub fn text<'a>(&self, source: &'a str, arena: &GreenNodeArena) -> &'a str
pub fn text<'a>(&self, source: &'a str, arena: &GreenNodeArena) -> &'a str
Get the text of this node from the source