pub struct DynPluginFunctionInfo { /* private fields */ }Expand description
Wrapper for dynamically loaded plugin runtime functions.
This implements the MachineFunction trait by calling the FFI bridge.
Implementations
Sourceimpl DynPluginFunctionInfo
impl DynPluginFunctionInfo
Sourcepub unsafe fn new(
name: Symbol,
instance: *mut PluginInstance,
function_fn: PluginFunctionFn,
) -> Self
pub unsafe fn new( name: Symbol, instance: *mut PluginInstance, function_fn: PluginFunctionFn, ) -> Self
Create a new dynamic plugin function wrapper.
Safety
instancemust be a valid pointer to the plugin instancefunction_fnmust be a valid function pointer- Both must remain valid for the lifetime of this struct