pub struct DynPluginMacroInfo { /* private fields */ }Expand description
Wrapper for dynamically loaded plugin macro functions.
This implements the MacroFunction trait by calling the FFI bridge
and serializing/deserializing arguments and results.
Implementations
Sourceimpl DynPluginMacroInfo
impl DynPluginMacroInfo
Sourcepub unsafe fn new(
name: Symbol,
ty: TypeNodeId,
instance: *mut PluginInstance,
macro_fn: PluginMacroFn,
) -> Self
pub unsafe fn new( name: Symbol, ty: TypeNodeId, instance: *mut PluginInstance, macro_fn: PluginMacroFn, ) -> Self
Create a new dynamic plugin macro wrapper.
Safety
instancemust be a valid pointer to the plugin instancemacro_fnmust be a valid function pointer for the macro- Both must remain valid for the lifetime of this struct