pub enum Instruction {
Show 72 variants
Move(Reg, Reg),
MoveConst(Reg, ConstPos),
MoveImmF(Reg, HFloat),
MoveRange(Reg, Reg, TypeSize),
Call(Reg, u8, TypeSize),
CallCls(Reg, u8, TypeSize),
CallExtFun(Reg, u8, TypeSize),
Closure(Reg, Reg),
Close(Reg),
MakeHeapClosure(Reg, Reg, TypeSize),
CloseHeapClosure(Reg),
CloneHeap(Reg),
CallIndirect(Reg, u8, TypeSize),
BoxAlloc(Reg, Reg, TypeSize),
BoxLoad(Reg, Reg, TypeSize),
BoxClone(Reg),
BoxRelease(Reg),
BoxStore(Reg, Reg, TypeSize),
CloneUserSum(Reg, TypeSize, TypeTableIndex),
ReleaseUserSum(Reg, TypeSize, TypeTableIndex),
GetUpValue(Reg, Reg, TypeSize),
SetUpValue(Reg, Reg, TypeSize),
GetGlobal(Reg, GlobalPos, TypeSize),
SetGlobal(GlobalPos, Reg, TypeSize),
GetState(Reg, TypeSize),
SetState(Reg, TypeSize),
PushStatePos(StateOffset),
PopStatePos(StateOffset),
Return0,
Return(Reg, TypeSize),
Delay(Reg, Reg, Reg),
Mem(Reg, Reg),
Jmp(Offset),
JmpIfNeg(Reg, Offset),
JmpTable(Reg, u8),
AddF(Reg, Reg, Reg),
SubF(Reg, Reg, Reg),
MulF(Reg, Reg, Reg),
DivF(Reg, Reg, Reg),
ModF(Reg, Reg, Reg),
NegF(Reg, Reg),
AbsF(Reg, Reg),
SqrtF(Reg, Reg),
SinF(Reg, Reg),
CosF(Reg, Reg),
PowF(Reg, Reg, Reg),
LogF(Reg, Reg),
AddI(Reg, Reg, Reg),
SubI(Reg, Reg, Reg),
MulI(Reg, Reg, Reg),
DivI(Reg, Reg, Reg),
ModI(Reg, Reg, Reg),
NegI(Reg, Reg),
AbsI(Reg, Reg),
PowI(Reg, Reg, Reg),
LogI(Reg, Reg, Reg),
Not(Reg, Reg),
Eq(Reg, Reg, Reg),
Ne(Reg, Reg, Reg),
Gt(Reg, Reg, Reg),
Ge(Reg, Reg, Reg),
Lt(Reg, Reg, Reg),
Le(Reg, Reg, Reg),
And(Reg, Reg, Reg),
Or(Reg, Reg, Reg),
CastFtoI(Reg, Reg),
CastItoF(Reg, Reg),
CastItoB(Reg, Reg),
AllocArray(Reg, Reg, TypeSize),
GetArrayElem(Reg, Reg, Reg),
SetArrayElem(Reg, Reg, Reg),
Dummy,
}Expand description
Instructions for bytecode. Currently, each instructon has the 64 bit size(Tag, up to 3 bytes arguments.)
Variants
Move(Reg, Reg)
Move Single Value, Destination, Source
MoveConst(Reg, ConstPos)
Load Single Value from Constants. Destination, Source
MoveImmF(Reg, HFloat)
Load Immediate float from half precision. Destination, Value
MoveRange(Reg, Reg, TypeSize)
Call(Reg, u8, TypeSize)
Call to internal function Function Address,Nargs,Word Size of Return Value
CallCls(Reg, u8, TypeSize)
CallExtFun(Reg, u8, TypeSize)
Call external rust functions or closure,
Currently, The execution branches into the invocation of a raw function item(function pointer), or Rust Closure depending on the information on the VM.
Previously there was another operation CallExtCls for Rust Closure Invocation separately but keeping distinction between raw function pointer and closure at the type checking and program generation stage, makes compiler’s design complex thus it was removed.
The distincion may also be resolved statically at linking time.
Function Address,Nargs,Nret
Closure(Reg, Reg)
Create new closure. Destination, index of inner function prototype in global function table.
Close(Reg)
register of the closure to be closed. other local closures will be released with this instruction.
MakeHeapClosure(Reg, Reg, TypeSize)
Create a closure on the heap. Destination, Function Index, Closure Size The closure data is allocated on the heap and a HeapIdx is stored in the destination register.
CloseHeapClosure(Reg)
Close upvalues of a heap-based closure. Address Register (HeapIdx)
CloneHeap(Reg)
Increment reference count of a heap object and register it for release at scope exit. Used for call-by-value cloning of heap-allocated closures.
CallIndirect(Reg, u8, TypeSize)
Call a closure indirectly through heap storage. Function register (HeapIdx), nargs, nret
BoxAlloc(Reg, Reg, TypeSize)
Box a value by allocating it on the heap. BoxAlloc(destination_for_HeapIdx, source_value, inner_word_size) Allocates a heap object of inner_word_size, copies from source, stores HeapIdx in destination.
BoxLoad(Reg, Reg, TypeSize)
Unbox a heap-allocated value by loading from heap. BoxLoad(destination, source_HeapIdx, inner_word_size) Reads inner_word_size words from heap and stores to destination registers.
BoxClone(Reg)
Increment the reference count of a boxed heap object. BoxClone(heap_ptr_reg)
BoxRelease(Reg)
Decrement the reference count of a boxed heap object and free if count reaches 0. BoxRelease(heap_ptr_reg)
BoxStore(Reg, Reg, TypeSize)
Write a value into an already-allocated heap object (destructive update). BoxStore(heap_ptr_reg, src_reg, inner_word_size)
CloneUserSum(Reg, TypeSize, TypeTableIndex)
Clone all boxed references within a UserSum value. CloneUserSum(value_reg, value_size, type_table_index) The type_table_index is used to look up TypeNodeId from Program.type_table.
ReleaseUserSum(Reg, TypeSize, TypeTableIndex)
Release all boxed references within a UserSum value. ReleaseUserSum(value_reg, value_size, type_table_index) The type_table_index is used to look up TypeNodeId from Program.type_table.
GetUpValue(Reg, Reg, TypeSize)
destination,source, size
SetUpValue(Reg, Reg, TypeSize)
GetGlobal(Reg, GlobalPos, TypeSize)
destination,source
SetGlobal(GlobalPos, Reg, TypeSize)
GetState(Reg, TypeSize)
Call internal state over time, destination,source
SetState(Reg, TypeSize)
PushStatePos(StateOffset)
PopStatePos(StateOffset)
Return0
Return from current function without return value.
Return(Reg, TypeSize)
value start position, Nrets
Delay(Reg, Reg, Reg)
Mem(Reg, Reg)
Jmp(Offset)
jump to instruction over the offset.
JmpIfNeg(Reg, Offset)
jump to instruction over the offset if the value in the first argument was negative.
JmpTable(Reg, u8)
Jump table for switch/match expressions. (scrutinee_reg, table_index) Looks up the scrutinee value in the jump table and jumps to the corresponding offset. If not found, jumps to the default offset stored in the jump table.
AddF(Reg, Reg, Reg)
Primitive Operations. Destination, Src1, Src2
SubF(Reg, Reg, Reg)
MulF(Reg, Reg, Reg)
DivF(Reg, Reg, Reg)
ModF(Reg, Reg, Reg)
NegF(Reg, Reg)
AbsF(Reg, Reg)
SqrtF(Reg, Reg)
SinF(Reg, Reg)
CosF(Reg, Reg)
PowF(Reg, Reg, Reg)
LogF(Reg, Reg)
AddI(Reg, Reg, Reg)
SubI(Reg, Reg, Reg)
MulI(Reg, Reg, Reg)
DivI(Reg, Reg, Reg)
ModI(Reg, Reg, Reg)
NegI(Reg, Reg)
AbsI(Reg, Reg)
PowI(Reg, Reg, Reg)
LogI(Reg, Reg, Reg)
Not(Reg, Reg)
Eq(Reg, Reg, Reg)
Ne(Reg, Reg, Reg)
Gt(Reg, Reg, Reg)
Ge(Reg, Reg, Reg)
Lt(Reg, Reg, Reg)
Le(Reg, Reg, Reg)
And(Reg, Reg, Reg)
Or(Reg, Reg, Reg)
CastFtoI(Reg, Reg)
CastItoF(Reg, Reg)
CastItoB(Reg, Reg)
AllocArray(Reg, Reg, TypeSize)
Allocate new array in the heap. Destination, size of array, type size of each element
GetArrayElem(Reg, Reg, Reg)
Get array element: destination, array, index
SetArrayElem(Reg, Reg, Reg)
Set array element. Because the array is immutable, this instruction is mostly used for initialization. array, index, value.
Dummy
Dummy instruction for testing