Machine

Struct Machine 

pub struct Machine {
    pub prog: Program,
    pub closures: ClosureStorage,
    pub heap: HeapStorage,
    pub ext_fun_table: Vec<(Symbol, ExtFunType)>,
    pub ext_cls_table: Vec<(Symbol, ExtClsType)>,
    pub arrays: ArrayStorage,
    /* private fields */
}
The virtual machine that executes mimium bytecode programs.

A Machine holds the compiled Program, value stack and all live closures. External functions and closures installed from plugins are also managed here.

Fields

prog: Programclosures: ClosureStorageheap: HeapStorageext_fun_table: Vec<(Symbol, ExtFunType)>ext_cls_table: Vec<(Symbol, ExtClsType)>arrays: ArrayStorage

Implementations

Source

impl Machine

pub fn drop_closure(&mut self, id: ClosureIdx)

Drop a closure by decrementing its reference count. When refcount reaches 0, recursively drops captured closures and removes the closure.

pub fn new( prog: Program, extfns: impl Iterator<Item = ExtFunInfo>, extcls: impl Iterator<Item = Box<dyn MachineFunction>>, ) -> Self

Create a new VM from a compiled Program and external functions.

pub fn new_resume(&self, prog: Program) -> Self

Create a new VM instance with the new program, preserving the current state as possible.

pub fn clear_stack(&mut self)

pub fn get_stack(&self, offset: i64) -> RawVal

pub fn get_stack_range( &self, offset: i64, word_size: TypeSize, ) -> (Range<usize>, &[RawVal])

pub fn get_stack_range_mut( &mut self, offset: i64, word_size: TypeSize, ) -> (Range<usize>, &mut [RawVal])

pub fn set_stack(&mut self, offset: i64, v: RawVal)

pub fn set_stack_range(&mut self, offset: i64, vs: &[RawVal])

pub fn get_top_n(&self, n: usize) -> &[RawVal]

pub fn get_current_ext_call_nargs(&self) -> u8

Number of argument words for the currently executing external function call.

pub fn get_current_ext_call_idx(&self) -> Option<usize>

pub fn get_closure_idx_from_heap(&self, heap_idx: HeapIdx) -> ClosureIdx

Extract the ClosureIdx stored inside a heap-allocated closure object.

During the migration period the heap object wraps a single ClosureIdx in data[0]. External callers (e.g. the scheduler plugin) can use this to obtain the underlying closure index from a HeapIdx value that lives on the VM stack.

pub fn get_open_upvalue( &self, upper_base: usize, ov: OpenUpValue, ) -> (Range<usize>, &[RawVal])

pub fn get_closure(&self, idx: ClosureIdx) -> &Closure

pub fn get_as<T>(v: RawVal) -> T

pub fn get_as_array<T>(v: &[RawVal]) -> &[T]

pub fn to_value<T>(v: T) -> RawVal

pub fn wrap_extern_cls(&mut self, extcls: ExtClsInfo) -> ClosureIdx

This API is used for defining higher-order external function that returns some external rust closure. Because the native closure cannot be called with CallCls directly, the vm appends an additional function the program, that wraps external closure call with an internal closure.

pub fn execute( &mut self, func_i: usize, cls_i: Option<ClosureIdx>, ) -> ReturnCode

Execute a function within the VM.

func_i is an index into the program’s function table and cls_i is an optional closure that provides the environment for the call. The returned ReturnCode is the number of values pushed on the stack as a result of the call.

pub fn install_extern_fn(&mut self, name: Symbol, f: ExtFunType) -> usize

pub fn install_extern_cls(&mut self, name: Symbol, f: ExtClsType) -> usize

pub fn alloc_code(&mut self, expr: ExprNodeId) -> RawVal

Store a code value (AST fragment) and return a RawVal index.

The returned value can later be passed to [get_code] to recover the original ExprNodeId.

pub fn get_code(&self, val: RawVal) -> ExprNodeId

Retrieve a previously stored code value by its RawVal index.

pub fn try_get_code(&self, val: RawVal) -> Option<ExprNodeId>

Try retrieving a previously stored code value by its RawVal index.

pub fn execute_idx(&mut self, idx: usize) -> ReturnCode

pub fn execute_entry(&mut self, entry: &str) -> ReturnCode

pub fn execute_main(&mut self) -> ReturnCode